@@ -60,6 +60,9 @@ def main(argv):
     gc.collect()
     gc.collect()
 
+    if lib.Cryptography_HAS_OPENSSL_CLEANUP:
+        lib.OPENSSL_cleanup()
+
     # Swap back to the original functions so that if OpenSSL tries to free
     # something from its atexit handle it won't be going through a Python
     # function, which will be deallocated when this function returns
@@ -214,3 +217,19 @@ class TestOpenSSLMemoryLeaks(object):
                 )
             ).private_key(backend)
         """))
+
+    def test_ec_derive_private_key(self):
+        assert_no_memory_leaks(textwrap.dedent("""
+        def func():
+            from cryptography.hazmat.backends.openssl import backend
+            from cryptography.hazmat.primitives.asymmetric import ec
+            ec.derive_private_key(1, ec.SECP256R1(), backend)
+        """))
+
+    def test_x25519_pubkey_from_private_key(self):
+        assert_no_memory_leaks(textwrap.dedent("""
+        def func():
+            from cryptography.hazmat.primitives.asymmetric import x25519
+            private_key = x25519.X25519PrivateKey.generate()
+            private_key.public_key()
+        """))
