@@ -20,12 +20,13 @@ from cryptography.hazmat.primitives.asymmetric import ec
 from cryptography.hazmat.primitives.asymmetric.utils import (
     Prehashed, encode_dss_signature
 )
+from cryptography.utils import CryptographyDeprecationWarning
 
 from .fixtures_ec import EC_KEY_SECP384R1
 from ...doubles import DummyKeySerializationEncryption
 from ...utils import (
     load_fips_ecdsa_key_pair_vectors, load_fips_ecdsa_signing_vectors,
-    load_kasvs_ecdh_vectors, load_vectors_from_file,
+    load_kasvs_ecdh_vectors, load_nist_vectors, load_vectors_from_file,
     raises_unsupported_algorithm
 )
 
@@ -64,8 +65,8 @@ def _skip_exchange_algorithm_unsupported(backend, algorithm, curve):
         algorithm, curve
     ):
         pytest.skip(
-            "Exchange algorithm is not supported by this backend {0}".format(
-                backend
+            "Exchange with {0} curve is not supported by {1}".format(
+                curve.name, backend
             )
         )
 
@@ -141,43 +142,26 @@ def test_ec_numbers():
     assert numbers.public_numbers.y == 3
     assert isinstance(numbers.public_numbers.curve, DummyCurve)
 
+
+@pytest.mark.parametrize(
+    ("private_value", "x", "y", "curve"),
+    [
+        (None, 2, 3, DummyCurve()),
+        (1, None, 3, DummyCurve()),
+        (1, 2, None, DummyCurve()),
+        (1, 2, 3, None),
+    ]
+)
+def test_invalid_ec_numbers_args(private_value, x, y, curve):
     with pytest.raises(TypeError):
         ec.EllipticCurvePrivateNumbers(
-            None,
-            ec.EllipticCurvePublicNumbers(
-                2, 3, DummyCurve()
-            )
+            private_value, ec.EllipticCurvePublicNumbers(x, y, curve)
         )
 
-    with pytest.raises(TypeError):
-        ec.EllipticCurvePrivateNumbers(
-            1,
-            ec.EllipticCurvePublicNumbers(
-                None, 3, DummyCurve()
-            )
-        )
 
+def test_invalid_private_numbers_public_numbers():
     with pytest.raises(TypeError):
-        ec.EllipticCurvePrivateNumbers(
-            1,
-            ec.EllipticCurvePublicNumbers(
-                2, None, DummyCurve()
-            )
-        )
-
-    with pytest.raises(TypeError):
-        ec.EllipticCurvePrivateNumbers(
-            1,
-            ec.EllipticCurvePublicNumbers(
-                2, 3, None
-            )
-        )
-
-    with pytest.raises(TypeError):
-        ec.EllipticCurvePrivateNumbers(
-            1,
-            None
-        )
+        ec.EllipticCurvePrivateNumbers(1, None)
 
 
 def test_encode_point():
@@ -350,13 +334,13 @@ class TestECDSAVectors(object):
         pkey = key.public_key()
         assert pkey
 
-        signer = pytest.deprecated_call(key.signer, ec.ECDSA(hash_type()))
+        with pytest.warns(CryptographyDeprecationWarning):
+            signer = key.signer(ec.ECDSA(hash_type()))
         signer.update(b"YELLOW SUBMARINE")
         signature = signer.finalize()
 
-        verifier = pytest.deprecated_call(
-            pkey.verifier, signature, ec.ECDSA(hash_type())
-        )
+        with pytest.warns(CryptographyDeprecationWarning):
+            verifier = pkey.verifier(signature, ec.ECDSA(hash_type()))
         verifier.update(b"YELLOW SUBMARINE")
         verifier.verify()
 
@@ -513,12 +497,11 @@ class TestECDSAVectors(object):
 
         signature = encode_dss_signature(vector['r'], vector['s'])
 
-        verifier = key.verifier(
+        key.verify(
             signature,
+            vector['message'],
             ec.ECDSA(hash_type())
         )
-        verifier.update(vector['message'])
-        verifier.verify()
 
     @pytest.mark.parametrize(
         "vector",
@@ -542,17 +525,19 @@ class TestECDSAVectors(object):
 
         signature = encode_dss_signature(vector['r'], vector['s'])
 
-        verifier = key.verifier(
-            signature,
-            ec.ECDSA(hash_type())
-        )
-        verifier.update(vector['message'])
-
         if vector["fail"] is True:
             with pytest.raises(exceptions.InvalidSignature):
-                verifier.verify()
+                key.verify(
+                    signature,
+                    vector['message'],
+                    ec.ECDSA(hash_type())
+                )
         else:
-            verifier.verify()
+            key.verify(
+                signature,
+                vector['message'],
+                ec.ECDSA(hash_type())
+            )
 
     def test_sign(self, backend):
         _skip_curve_unsupported(backend, ec.SECP256R1())
@@ -1066,9 +1051,9 @@ class TestECDH(object):
                 ec._CURVE_TYPES[vector['curve']]()
             )
         )
-        # Errno 5 and 6 indicates a bad public key, this doesn't test the ECDH
-        # code at all
-        if vector['fail'] and vector['errno'] in [5, 6]:
+        # Errno 5-7 indicates a bad public or private key, this doesn't test
+        # the ECDH code at all
+        if vector['fail'] and vector['errno'] in [5, 6, 7]:
             with pytest.raises(ValueError):
                 private_numbers.private_key(backend)
             return
@@ -1095,10 +1080,39 @@ class TestECDH(object):
         # At this point fail indicates that one of the underlying keys was
         # changed. This results in a non-matching derived key.
         if vector['fail']:
+            # Errno 8 indicates Z should be changed.
+            assert vector['errno'] == 8
             assert z != vector['Z']
         else:
             assert z == vector['Z']
 
+    @pytest.mark.parametrize(
+        "vector",
+        load_vectors_from_file(
+            os.path.join("asymmetric", "ECDH", "brainpool.txt"),
+            load_nist_vectors
+        )
+    )
+    def test_brainpool_kex(self, backend, vector):
+        curve = ec._CURVE_TYPES[vector['curve'].decode('ascii')]
+        _skip_exchange_algorithm_unsupported(backend, ec.ECDH(), curve)
+        key = ec.EllipticCurvePrivateNumbers(
+            int(vector['da'], 16),
+            ec.EllipticCurvePublicNumbers(
+                int(vector['x_qa'], 16), int(vector['y_qa'], 16), curve()
+            )
+        ).private_key(backend)
+        peer = ec.EllipticCurvePrivateNumbers(
+            int(vector['db'], 16),
+            ec.EllipticCurvePublicNumbers(
+                int(vector['x_qb'], 16), int(vector['y_qb'], 16), curve()
+            )
+        ).private_key(backend)
+        shared_secret = key.exchange(ec.ECDH(), peer.public_key())
+        assert shared_secret == binascii.unhexlify(vector["x_z"])
+        shared_secret_2 = peer.exchange(ec.ECDH(), key.public_key())
+        assert shared_secret_2 == binascii.unhexlify(vector["x_z"])
+
     def test_exchange_unsupported_algorithm(self, backend):
         _skip_curve_unsupported(backend, ec.SECP256R1())
 
